'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.TRANSITION_CANCEL = exports.TRANSITION_START = exports.TRANSITION_ERROR = exports.TRANSITION_SUCCESS = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _xstream = require('xstream');

var _xstream2 = _interopRequireDefault(_xstream);

var _dropRepeats = require('xstream/extra/dropRepeats');

var _dropRepeats2 = _interopRequireDefault(_dropRepeats);

var _router5TransitionPath = require('router5-transition-path');

var _router5TransitionPath2 = _interopRequireDefault(_router5TransitionPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TRANSITION_SUCCESS = '@@router5/TRANSITION_SUCCESS';
var TRANSITION_ERROR = '@@router5/TRANSITION_ERROR';
var TRANSITION_START = '@@router5/TRANSITION_START';
var TRANSITION_CANCEL = '@@router5/TRANSITION_CANCEL';

exports.TRANSITION_SUCCESS = TRANSITION_SUCCESS;
exports.TRANSITION_ERROR = TRANSITION_ERROR;
exports.TRANSITION_START = TRANSITION_START;
exports.TRANSITION_CANCEL = TRANSITION_CANCEL;


function xsPluginFactory(listener) {
    return function xsPlugin() {
        var dispatch = function dispatch(type, isError) {
            return function (toState, fromState, error) {
                if (listener) {
                    var routerEvt = { type: type, toState: toState, fromState: fromState };

                    listener.next(isError ? _extends({}, routerEvt, { error: error }) : routerEvt);
                }
            };
        };

        return {
            onStop: function onStop() {
                return listener.complete();
            },
            onTransitionSuccess: dispatch(TRANSITION_SUCCESS),
            onTransitionError: dispatch(TRANSITION_ERROR, true),
            onTransitionStart: dispatch(TRANSITION_START),
            onTransitionCancel: dispatch(TRANSITION_CANCEL)
        };
    };
}

function createObservables(router) {
    // Events observable
    var transitionEvents$ = _xstream2.default.create({
        start: function start(listener) {
            router.usePlugin(xsPluginFactory(listener));
        },
        stop: function stop() {}
    });

    // Transition Route
    var transitionRoute$ = transitionEvents$.map(function (_) {
        return _.type === TRANSITION_START ? _.toState : null;
    }).startWith(null);

    // Error
    var transitionError$ = transitionEvents$.filter(function (_) {
        return _.type;
    }).map(function (_) {
        return _.type === TRANSITION_ERROR ? _.error : null;
    }).startWith(null).compose((0, _dropRepeats2.default)());

    // Route with intersection
    var routeState$ = transitionEvents$.filter(function (_) {
        return _.type === TRANSITION_SUCCESS && _.toState !== null;
    }).map(function (_ref) {
        var toState = _ref.toState,
            fromState = _ref.fromState;

        var _transitionPath = (0, _router5TransitionPath2.default)(toState, fromState),
            intersection = _transitionPath.intersection;

        return { intersection: intersection, route: toState };
    }).startWith({ intersection: '', route: router.getState() });

    // Create a route observable
    var route$ = routeState$.map(function (_ref2) {
        var route = _ref2.route;
        return route;
    });

    // Create a route node observable
    var routeNode = function routeNode(node) {
        return routeState$.filter(function (_ref3) {
            var intersection = _ref3.intersection;
            return intersection === node;
        }).map(function (_ref4) {
            var route = _ref4.route;
            return route;
        }).startWith(router.getState());
    };

    // Return observables
    return {
        route$: route$,
        routeNode: routeNode,
        transitionError$: transitionError$,
        transitionRoute$: transitionRoute$
    };
}

exports.default = createObservables;